@@ -1,4 +1,7 @@
-use std::{collections::BTreeMap, rc::Rc};
+use std::{
+    collections::{BTreeMap, HashMap},
+    rc::Rc,
+};
 
 use prost::bytes::Bytes;
 use prost_types::{
@@ -86,22 +89,6 @@ impl TypeMap {
             .map(|(&number, field)| (field.json_name.clone(), number))
             .collect();
 
-        let reserved_ranges = message_proto
-            .reserved_range
-            .iter()
-            .map(|n| (n.start() as u32)..(n.end() as u32))
-            .collect();
-        let reserved_names = message_proto
-            .reserved_name
-            .iter()
-            .map(|n| n.as_str().into())
-            .collect();
-        let extension_ranges = message_proto
-            .extension_range
-            .iter()
-            .map(|n| (n.start() as u32)..(n.end() as u32))
-            .collect();
-
         if is_map_entry
             && (!fields.contains_key(&MAP_ENTRY_KEY_NUMBER)
                 || !fields.contains_key(&MAP_ENTRY_VALUE_NUMBER))
@@ -123,9 +110,6 @@ impl TypeMap {
             full_name,
             parent,
             is_map_entry,
-            reserved_names,
-            reserved_ranges,
-            extension_ranges,
             extensions: vec![],
         });
 
@@ -190,10 +174,9 @@ impl TypeMap {
                 field_descriptor_proto::Type::Enum => {
                     let enum_ty = self.get_enum(ty);
                     enum_ty
-                        .values
-                        .iter()
-                        .find(|(_, v)| *v.name == *value)
-                        .map(|(&n, _)| crate::Value::EnumNumber(n))
+                        .value_names
+                        .get(value.as_str())
+                        .map(|&index| crate::Value::EnumNumber(enum_ty.values[index].number))
                         .ok_or(())
                 }
                 field_descriptor_proto::Type::Message | field_descriptor_proto::Type::Group => {
@@ -264,54 +247,40 @@ impl TypeMap {
             syntax,
         }: EnumProto,
     ) -> Result<(), DescriptorError> {
-        let value_names = enum_proto
-            .value
-            .iter()
-            .map(|value| (value.name().to_owned(), value.number()))
-            .collect();
-
         let package_name = parse_namespace(&full_name);
-        let values: BTreeMap<_, _> = enum_proto
+        let mut values: Vec<_> = enum_proto
             .value
             .iter()
-            .map(|value_proto| {
-                (
-                    value_proto.number(),
-                    EnumValueDescriptorInner {
-                        name: value_proto.name().into(),
-                        full_name: make_full_name(package_name, value_proto.name()),
-                    },
-                )
+            .map(|value_proto| EnumValueDescriptorInner {
+                name: value_proto.name().into(),
+                number: value_proto.number(),
+                full_name: make_full_name(package_name, value_proto.name()),
             })
             .collect();
+        values.sort_by_key(|v| v.number);
+
+        let value_names: HashMap<Box<str>, usize> = values
+            .iter()
+            .enumerate()
+            .map(|(index, value)| (value.name.clone(), index))
+            .collect();
 
         let parent = parent.map(|p| self.try_get_by_name(&p)).transpose()?;
 
         let default_value = if syntax == Syntax::Proto2 {
-            enum_proto
+            let name = enum_proto
                 .value
                 .get(0)
-                .map(|v| v.number())
                 .ok_or_else(DescriptorError::empty_enum)?
+                .name();
+            value_names[name]
         } else {
-            if !values.contains_key(&0) {
-                return Err(DescriptorError::empty_enum());
-            }
-
-            0
+            values
+                .iter()
+                .position(|v| v.number == 0)
+                .ok_or_else(DescriptorError::empty_enum)?
         };
 
-        let reserved_ranges = enum_proto
-            .reserved_range
-            .iter()
-            .map(|n| n.start()..=n.end())
-            .collect();
-        let reserved_names = enum_proto
-            .reserved_name
-            .iter()
-            .map(|n| n.as_str().into())
-            .collect();
-
         debug_assert_eq!(
             self.get_by_name(&full_name),
             Some(TypeId::new_enum(self.enums.len()))
@@ -322,8 +291,6 @@ impl TypeMap {
             value_names,
             values,
             default_value,
-            reserved_ranges,
-            reserved_names,
         });
         Ok(())
     }
