@@ -2,9 +2,6 @@ mod build;
 #[cfg(test)]
 mod tests;
 
-use prost::encoding::WireType;
-use prost_types::field_descriptor_proto;
-
 use std::{
     collections::{
         hash_map::{self, HashMap},
@@ -15,6 +12,12 @@ use std::{
     ops::{Range, RangeInclusive},
 };
 
+use prost::encoding::WireType;
+use prost_types::{
+    field_descriptor_proto, DescriptorProto, EnumDescriptorProto, EnumValueDescriptorProto,
+    FieldDescriptorProto, OneofDescriptorProto,
+};
+
 use crate::descriptor::{
     debug_fmt_iter, make_full_name, parse_name, parse_namespace, DescriptorError, FileDescriptor,
     MAP_ENTRY_KEY_NUMBER, MAP_ENTRY_VALUE_NUMBER,
@@ -45,9 +48,6 @@ struct MessageDescriptorInner {
     field_names: HashMap<Box<str>, u32>,
     field_json_names: HashMap<Box<str>, u32>,
     oneof_decls: Box<[OneofDescriptorInner]>,
-    reserved_ranges: Box<[Range<u32>]>,
-    reserved_names: Box<[Box<str>]>,
-    extension_ranges: Box<[Range<u32>]>,
     extensions: Vec<usize>,
 }
 
@@ -109,22 +109,21 @@ pub struct EnumDescriptor {
 struct EnumDescriptorInner {
     full_name: Box<str>,
     parent: Option<TypeId>,
-    value_names: HashMap<String, i32>,
-    values: BTreeMap<i32, EnumValueDescriptorInner>,
-    default_value: i32,
-    reserved_ranges: Box<[RangeInclusive<i32>]>,
-    reserved_names: Box<[Box<str>]>,
+    value_names: HashMap<Box<str>, usize>,
+    values: Vec<EnumValueDescriptorInner>,
+    default_value: usize,
 }
 
 /// A value in a protobuf enum type.
 #[derive(Clone, PartialEq, Eq)]
 pub struct EnumValueDescriptor {
     parent: EnumDescriptor,
-    number: i32,
+    index: usize,
 }
 
 struct EnumValueDescriptorInner {
     name: Box<str>,
+    number: i32,
     full_name: Box<str>,
 }
 
@@ -240,6 +239,12 @@ impl MessageDescriptor {
         parse_namespace(&self.root_message_ty().full_name)
     }
 
+    /// Gets a reference to the raw [`DescriptorProto`] wrapped by this [`MessageDescriptor`].
+    pub fn descriptor_proto(&self) -> &DescriptorProto {
+        self.parent_file()
+            .find_message_descriptor_proto(self.package_name(), self.full_name())
+    }
+
     /// Gets an iterator yielding a [`FieldDescriptor`] for each field defined in this message.
     pub fn fields(&self) -> impl ExactSizeIterator<Item = FieldDescriptor> + '_ {
         self.message_ty()
@@ -333,17 +338,26 @@ impl MessageDescriptor {
 
     /// Gets an iterator over reserved field number ranges in this message.
     pub fn reserved_ranges(&self) -> impl ExactSizeIterator<Item = Range<u32>> + '_ {
-        self.message_ty().reserved_ranges.iter().cloned()
+        self.descriptor_proto()
+            .reserved_range
+            .iter()
+            .map(|n| (n.start() as u32)..(n.end() as u32))
     }
 
     /// Gets an iterator over reserved field names in this message.
     pub fn reserved_names(&self) -> impl ExactSizeIterator<Item = &str> + '_ {
-        self.message_ty().reserved_names.iter().map(Box::as_ref)
+        self.descriptor_proto()
+            .reserved_name
+            .iter()
+            .map(|n| n.as_ref())
     }
 
     /// Gets an iterator over extension field number ranges in this message.
     pub fn extension_ranges(&self) -> impl ExactSizeIterator<Item = Range<u32>> + '_ {
-        self.message_ty().extension_ranges.iter().cloned()
+        self.descriptor_proto()
+            .extension_range
+            .iter()
+            .map(|n| (n.start() as u32)..(n.end() as u32))
     }
 
     /// Gets an iterator over extensions to this message.
@@ -416,6 +430,16 @@ impl FieldDescriptor {
         &self.message_field_ty().full_name
     }
 
+    /// Gets a reference to the raw [`FieldDescriptorProto`] wrapped by this [`FieldDescriptor`].
+    pub fn field_descriptor_proto(&self) -> &FieldDescriptorProto {
+        self.parent_message()
+            .descriptor_proto()
+            .field
+            .iter()
+            .find(|field| field.number() as u32 == self.field)
+            .expect("field not found")
+    }
+
     /// Gets the unique number for this message field.
     pub fn number(&self) -> u32 {
         self.field
@@ -579,6 +603,28 @@ impl ExtensionDescriptor {
         }
     }
 
+    /// Gets a reference to the raw [`FieldDescriptorProto`] wrapped by this [`ExtensionDescriptor`].
+    pub fn field_descriptor_proto(&self) -> &FieldDescriptorProto {
+        let name = self.name();
+        let package = self.package_name();
+        let namespace = parse_namespace(self.full_name());
+
+        if namespace == package {
+            self.parent_file()
+                .filter_file_descriptor_protos_by_package(package)
+                .flat_map(|file| file.extension.iter())
+                .find(|extension| extension.name() == name)
+                .expect("extension not found")
+        } else {
+            self.parent_file()
+                .find_message_descriptor_proto(package, namespace)
+                .extension
+                .iter()
+                .find(|extension| extension.name() == name)
+                .expect("extension not found")
+        }
+    }
+
     /// Gets the number for this extension field.
     pub fn number(&self) -> u32 {
         self.extension_ty().number
@@ -823,11 +869,33 @@ impl EnumDescriptor {
         }
     }
 
+    /// Gets a reference to the raw [`EnumDescriptorProto`] wrapped by this [`EnumDescriptor`].
+    pub fn enum_descriptor_proto(&self) -> &EnumDescriptorProto {
+        let name = self.name();
+        let package = self.package_name();
+        let namespace = parse_namespace(self.full_name());
+
+        if namespace == package {
+            self.parent_file()
+                .filter_file_descriptor_protos_by_package(package)
+                .flat_map(|file| file.enum_type.iter())
+                .find(|extension| extension.name() == name)
+                .expect("enum not found")
+        } else {
+            self.parent_file()
+                .find_message_descriptor_proto(package, namespace)
+                .enum_type
+                .iter()
+                .find(|enum_ty| enum_ty.name() == name)
+                .expect("enum not found")
+        }
+    }
+
     /// Gets the default value for the enum type.
     pub fn default_value(&self) -> EnumValueDescriptor {
         EnumValueDescriptor {
             parent: self.clone(),
-            number: self.enum_ty().default_value,
+            index: self.enum_ty().default_value,
         }
     }
 
@@ -836,42 +904,52 @@ impl EnumDescriptor {
         self.enum_ty()
             .value_names
             .get(name)
-            .map(|&number| EnumValueDescriptor {
+            .map(|&index| EnumValueDescriptor {
                 parent: self.clone(),
-                number,
+                index,
             })
     }
 
     /// Gets a [`EnumValueDescriptor`] for the enum value with the given number, or `None` if no such value exists.
+    ///
+    /// If the enum was defined with the `allow_alias` option and has multiple values with the given number, it is
+    /// unspecified which one will be returned.
     pub fn get_value(&self, number: i32) -> Option<EnumValueDescriptor> {
-        self.enum_ty()
+        match self
+            .enum_ty()
             .values
-            .get(&number)
-            .map(|_| EnumValueDescriptor {
+            .binary_search_by_key(&number, |v| v.number)
+        {
+            Ok(index) => Some(EnumValueDescriptor {
                 parent: self.clone(),
-                number,
-            })
+                index,
+            }),
+            Err(_) => None,
+        }
     }
 
     /// Gets an iterator yielding a [`EnumValueDescriptor`] for each value in this enum.
     pub fn values(&self) -> impl ExactSizeIterator<Item = EnumValueDescriptor> + '_ {
-        self.enum_ty()
-            .values
-            .keys()
-            .map(move |&number| EnumValueDescriptor {
-                parent: self.clone(),
-                number,
-            })
+        (0..self.enum_ty().values.len()).map(move |index| EnumValueDescriptor {
+            parent: self.clone(),
+            index,
+        })
     }
 
     /// Gets an iterator over reserved value number ranges in this enum.
     pub fn reserved_ranges(&self) -> impl ExactSizeIterator<Item = RangeInclusive<i32>> + '_ {
-        self.enum_ty().reserved_ranges.iter().cloned()
+        self.enum_descriptor_proto()
+            .reserved_range
+            .iter()
+            .map(|n| n.start()..=n.end())
     }
 
     /// Gets an iterator over reserved value names in this enum.
     pub fn reserved_names(&self) -> impl ExactSizeIterator<Item = &str> + '_ {
-        self.enum_ty().reserved_names.iter().map(Box::as_ref)
+        self.enum_descriptor_proto()
+            .reserved_name
+            .iter()
+            .map(|n| n.as_ref())
     }
 
     fn enum_ty(&self) -> &EnumDescriptorInner {
@@ -925,13 +1003,23 @@ impl EnumValueDescriptor {
         &self.enum_value_ty().full_name
     }
 
+    /// Gets a reference to the raw [`EnumValueDescriptorProto`] wrapped by this [`EnumValueDescriptor`].
+    pub fn enum_value_descriptor_proto(&self) -> &EnumValueDescriptorProto {
+        self.parent_enum()
+            .enum_descriptor_proto()
+            .value
+            .iter()
+            .find(|value| value.name() == self.name())
+            .expect("enum value not found")
+    }
+
     /// Gets the number representing this enum value.
     pub fn number(&self) -> i32 {
-        self.number
+        self.enum_value_ty().number
     }
 
     fn enum_value_ty(&self) -> &EnumValueDescriptorInner {
-        self.parent.enum_ty().values.get(&self.number).unwrap()
+        &self.parent.enum_ty().values[self.index]
     }
 }
 
@@ -966,6 +1054,11 @@ impl OneofDescriptor {
         &self.oneof_ty().full_name
     }
 
+    /// Gets a reference to the raw [`OneofDescriptorProto`] wrapped by this [`OneofDescriptor`].
+    pub fn oneof_descriptor_proto(&self) -> &OneofDescriptorProto {
+        &self.parent_message().descriptor_proto().oneof_decl[self.index]
+    }
+
     /// Gets an iterator yielding a [`FieldDescriptor`] for each field of the parent message this oneof contains.
     pub fn fields(&self) -> impl ExactSizeIterator<Item = FieldDescriptor> + '_ {
         self.oneof_ty()
